<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
version="2.0"
xmlns:xs="http://www.w3.org/2001/XMLSchema"
xmlns:f="http://local/functions"
xmlns:cat="http://www.saxonica.com/ns/doc/catalog"

exclude-result-prefixes="xs f cat"
xpath-default-namespace="http://www.saxonica.com/ns/doc/functions">

<!-- Performs minimal transform to migrate function-data2.xml ready for cetree.xsl:

     1. Add 'id' and 'title' attribute to top-level functions element
     2. Convert xref element links to 'href' attribute links that uses simple path-separators
        instead of terms like 'page' and 'subpage'
-->

<xsl:variable name="sections" select="doc('../src/catalog.xml')/cat:catalog/cat:section/@ref"/>

<xsl:variable name="docs" as="document-node()*">
<xsl:for-each select="$sections">
<xsl:variable name="doc" select="document(concat(., '.xml'),/)"/>
<xsl:sequence select="$doc"/>
</xsl:for-each>      
</xsl:variable>

<xsl:template match="node()|@*">
<xsl:copy>
<xsl:apply-templates select="node()|@*"/>
</xsl:copy>
</xsl:template>

<xsl:template match="functions">
<xsl:copy>
<xsl:apply-templates select="@*"/>
<xsl:attribute name="id" select="'functions'"/>
<xsl:attribute name="title" select="'Saxon Function Library'"/>
<xsl:apply-templates select="node()"/>
</xsl:copy>
</xsl:template>

<xsl:template match="xref" xmlns="" xpath-default-namespace="">
<xsl:variable name="prefix" select="'../'"/>

<a>
<xsl:attribute name="href"
select="if (exists(@section)) then
    concat('/', string-join((@section, @page, @subpage),'/'))
else if (exists(@page)) then
    concat($prefix, string-join((@page, @subpage),'/'))
else
    concat($prefix, @subpage)
"/>
<xsl:value-of select="."/>
<xsl:if test=".=''">
<xsl:choose>
<xsl:when test="@subpage and @section">
<xsl:value-of select="$docs/section[@xml:id=@section]/page[@xml:id=@page]/subpage[@xml:id=@subpage]/title"/>
</xsl:when>
<xsl:when test="@page and @section">
<xsl:value-of select="$docs/section[@xml:id=@section]/page[@xml:id=@page]/title"/>
</xsl:when>
<xsl:when test="@section">
<xsl:value-of select="$docs/section[@xml:id=@section]/title"/>
</xsl:when>
<xsl:otherwise>
<xsl:value-of select="@subpage"/>
</xsl:otherwise>
</xsl:choose>
</xsl:if>
</a>
</xsl:template>

</xsl:stylesheet>
