<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:cat="http://www.saxonica.com/ns/doc/catalog"
    xmlns:fn="http://www.saxonica.com/ns/doc/functions" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:saxon="http://saxon.sf.net/" exclude-result-prefixes="cat fn xs saxon">

    <!--<xsl:import href="preprocess-functions.xsl"/>-->

    <xsl:output method="xml" encoding="utf-8" indent="yes" saxon:suppress-indentation="p pre code"
        cdata-section-elements="samp"/>

    <xsl:param name="destination" select="'online'" as="xs:string"/>
    <!-- set this to "online" for the web site, "offline" for the document supplied in the download"-->

    <xsl:variable name="img" select="'img'"/>

    <xsl:variable name="sections" select="/cat:catalog/cat:section/@ref"/>

    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>

    <xsl:variable name="docs" as="document-node()*">
        <xsl:for-each select="$sections[not(. = ('functions','index-entries'))]">
            <xsl:variable name="doc" select="document(concat(., '.xml'),/)"/>
            <xsl:sequence select="$doc"/>
        </xsl:for-each>
    </xsl:variable>

    <xsl:template match="cat:catalog">
        <xsl:for-each select="$docs">
            <xsl:message>Processing <xsl:value-of select="document-uri(.)"/></xsl:message>
            <xsl:result-document href="{concat(section/@xml:id,'.xml')}" method="xml" indent="yes"
                cdata-section-elements="samp" saxon:suppress-indentation="p pre code samp">
                <xsl:apply-templates select="section">
                    <xsl:with-param name="this-doc" select="section/@xml:id" tunnel="yes"/>
                </xsl:apply-templates>
            </xsl:result-document>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="index" priority="5">
        <dfn>
            <xsl:apply-templates/>
        </dfn>

    </xsl:template>

    <xsl:template match="*">
        <xsl:copy copy-namespaces="no">
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <!--
<xsl:template match="*">
<xsl:message> *** Unrecognized element <xsl:value-of select="name(.)"/> of type <xsl:value-of
select="saxon:type-annotation(.)"/> *** </xsl:message>
</xsl:template>
-->

    <xsl:template
        match="a[starts-with(string(@href), '../javadoc')
or starts-with(string(@href), '../dotnetdoc')]">
        <!-- redundant, same as the generic rule -->
        <xsl:copy copy-namespaces="no">
            <xsl:copy-of select="@* except (@class, @href)"/>
            <!--<xsl:attribute name="target" select="'_blank'"/>-->
            <xsl:attribute name="href" select="substring(@href, 4)"/>
            <xsl:attribute name="class" select="'apilink'"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="a">
        <xsl:copy copy-namespaces="no">
            <xsl:for-each select="@href">
                <xsl:attribute name="href"
                    select="if (not(starts-with(., 'http://')) and ends-with(.,'.xml')) then 
concat('../', substring(., 1, string-length(.) - 4))
else ."
                />
            </xsl:for-each>
            <xsl:copy-of select="@* except @href"/>
            <!--<xsl:attribute name="target" select="'_blank'"/>-->
            <xsl:attribute name="class" select="'bodylink'"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="@content[. eq 'para']"/>

    <xsl:template match="a[starts-with(string(@href), '#')]">
        <xsl:param name="this-doc" required="yes" tunnel="yes" as="xs:string"/>
        <xsl:message terminate="yes">** obsolete hyperlink in <xsl:value-of select="document-uri(/)"
            />: <xsl:copy-of select="."/></xsl:message>
    </xsl:template>

    <xsl:template match="a[matches(string(@href), '^[A-Za-z\-]+.html#.+$')]">
        <xsl:param name="this-doc" required="yes" tunnel="yes" as="xs:string"/>
        <xsl:message>** obsolete hyperlink in <xsl:value-of select="document-uri(/)"/>: <xsl:copy-of
                select="."/></xsl:message>
    </xsl:template>

    <xsl:template match="a[matches(string(@href), '^[A-Za-z\-]+\.html$')]">
        <xsl:param name="this-doc" required="yes" tunnel="yes" as="xs:string"/>
        <xsl:message>** obsolete hyperlink in <xsl:value-of select="document-uri(/)"/>: <xsl:copy-of
                select="."/></xsl:message>
    </xsl:template>

    <xsl:template match="section">
        <!--<xsl:call-template name="make-menu"/>-->
        <article id="{@xml:id}" title="{title}">
            <xsl:apply-templates select="page">
                <xsl:with-param name="parent" select="title"/>
            </xsl:apply-templates>
        </article>
    </xsl:template>

    <xsl:template match="page">
        <xsl:param name="this-doc" required="yes" tunnel="yes" as="xs:string"/>
        <xsl:param name="parent" select="()"/>
        <xsl:variable name="pgid" select="@xml:id"/>
        <xsl:if test="count(p) gt 10">
            <xsl:message>** Warning: long page <xsl:value-of select="$this-doc, @xml:id"
                /></xsl:message>
        </xsl:if>
        <xsl:call-template name="make-page">
            <xsl:with-param name="title" select="title"/>
            <xsl:with-param name="pgid" select="$pgid"/>
            <xsl:with-param name="parent" select="$parent"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="make-page">
        <xsl:param name="title" as="xs:string" required="yes"/>
        <xsl:param name="this-doc" as="xs:string" required="yes" tunnel="yes"/>
        <xsl:param name="pgid" as="xs:string" required="yes"/>
        <xsl:param name="subpageid" as="xs:string" required="no" select="''"/>
        <xsl:param name="parent" as="xs:string?" required="no" select="()"/>
        <xsl:variable name="path-prefix" as="xs:string"
            select="if (self::page) then '../'
else if (self::subpage) then '../../'
else ''"/>
        <xsl:variable name="paras" select=".//p except .//subpage//p"/>
        <xsl:variable name="est-length"
            select="max((ceiling(sum($paras/string-length()) idiv 4) + 
count($paras) * 20 +
(if (subpageindex) then count(subpage)*40 else 0), 1000))"/>
        <xsl:variable name="est-length"
            select="if (@size) then $est-length * @size else $est-length"/>
        <!--<xsl:comment>Est sizes <xsl:value-of select="sum($paras/string-length()) idiv 4, count($paras) * 20, 
(if (subpageindex) then count(subpage)*40 else 0)"/></xsl:comment>-->
        <xsl:choose>
            <xsl:when test="position() eq 1 and (empty($subpageid) or $subpageid eq '')">
                <h1>
                    <xsl:value-of
                        select="if ($title eq 'Introduction' and exists($parent)) then $parent
else $title"
                    />
                </h1>
                <xsl:apply-templates select="node()"/>
            </xsl:when>
            <xsl:otherwise>
                <section
                    id="{if (exists($subpageid) and $subpageid ne '') then $subpageid else $pgid}"
                    title="{$title}">
                    <h1>
                        <xsl:value-of select="$title"/>
                    </h1>
                    <xsl:apply-templates select="node()"/>
                </section>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="subpage">
        <xsl:param name="this-doc" required="yes" tunnel="yes" as="xs:string"/>
        <xsl:variable name="pgid" select="../@xml:id"/>
        <xsl:if test="count(p) gt 10">
            <xsl:message>** Warning: long sub-page <xsl:value-of select="$this-doc, $pgid, @xml:id"
                /></xsl:message>
        </xsl:if>
        <xsl:call-template name="make-page">
            <xsl:with-param name="title" select="title"/>
            <xsl:with-param name="pgid" select="$pgid"/>
            <xsl:with-param name="subpageid" select="@xml:id"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="subpageindex">
        <!-- placeholder to be filled dynamically: -->
        <nav>
            <ul/>
        </nav>
    </xsl:template>

    <xsl:template match="pageindex">
        <ul>
            <xsl:for-each select="../../page except parent::page">
                <li>
                    <p>
                        <a class="bodylink" href="{@xml:id}">
                            <xsl:value-of select="title"/>
                        </a>
                    </p>
                </li>
            </xsl:for-each>
        </ul>
    </xsl:template>

    <xsl:function name="fn:path-prefix" as="xs:string">
        <xsl:param name="node" as="element()"/>
        <xsl:sequence
            select="if ($node/(/table)) then '../' (: generated feature keys :)
else if ($node/ancestor::subpage) then '../../'
else if ($node/ancestor::page) then '../'
else ''"
        />
    </xsl:function>

    <xsl:template match="xref">
        <xsl:variable name="inFirstPage" as="xs:boolean"
            select="count(ancestor::page/preceding-sibling::page) eq 0"/>
        <xsl:variable name="inSubPage" as="xs:boolean" select="exists(ancestor::subpage)"/>
        <xsl:variable name="prefix"
            select="if($inSubPage) then
    if($inFirstPage) then '../' else '../../'
else if($inFirstPage) then '' else '../'"/>
        <xsl:variable name="promote" as="xs:boolean" select="exists(@page) and @page eq 'intro'"/>
        <xsl:variable name="sp-prefix" select="if($inSubPage) then '../' else ''"/>
        <xsl:variable name="pr-prefix"
            select="if($inSubPage) then
     '../../../'
else if($inFirstPage) then '' else '../'"/>
        <a class="bodylink">
            <xsl:attribute name="href"
                select="if ($promote) then
    if (exists(@section)) then
        concat('/', string-join((@section, @subpage),'/'))
    else
        concat($pr-prefix, @subpage)
else
    if (exists(@section)) then
        concat('/', string-join((@section, @page, @subpage),'/'))
    else if (exists(@page)) then
        concat($prefix, string-join((@page, @subpage),'/'))
    else
        concat($sp-prefix, @subpage)
"/>
            <xsl:value-of select="."/>
            <xsl:if test=".=''">
                <xsl:choose>
                    <xsl:when test="@subpage and @section">
                        <xsl:value-of
                            select="$docs/section[@xml:id=@section]/page[@xml:id=@page]/subpage[@xml:id=@subpage]/title"
                        />
                    </xsl:when>
                    <xsl:when test="@page and @section">
                        <xsl:value-of
                            select="$docs/section[@xml:id=@section]/page[@xml:id=@page]/title"/>
                    </xsl:when>
                    <xsl:when test="@section">
                        <xsl:value-of select="$docs/section[@xml:id=@section]/title"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="sp" select="@subpage"/>
                        <xsl:value-of select="ancestor::subpage/../subpage[@xml:id = $sp]/title"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
        </a>
    </xsl:template>

    <xsl:template match="code[@java]" priority="2">
        <a class="javalink" href="{@java}">
            <xsl:apply-templates/>
        </a>
    </xsl:template>

    <xsl:template match="title"/>

    <xsl:template match="subtitle">
        <h2 class="subtitle">
            <xsl:apply-templates/>
        </h2>
    </xsl:template>

    <xsl:template match="code[pre]">
        <code>
            <xsl:apply-templates select="pre/*"/>
        </code>
    </xsl:template>

    <xsl:template match="code[not(@java)]">
        <code>
            <xsl:apply-templates/>
        </code>
    </xsl:template>

    <xsl:template match="pre">
        <samp>
            <xsl:next-match/>
        </samp>
    </xsl:template>

    <xsl:template match="example" priority="2">
        <samp>
            <xsl:apply-templates/>
        </samp>
    </xsl:template>

    <xsl:template match="box" priority="2">
        <aside>
            <xsl:apply-templates/>
        </aside>
        <!--
<div class="boxed">
<xsl:apply-templates/>
</div>
-->
    </xsl:template>

    <xsl:template match="command" priority="2">
        <kbd>
            <xsl:apply-templates/>
        </kbd>
    </xsl:template>

    <xsl:template match="b" priority="2">
        <strong>
            <xsl:apply-templates/>
        </strong>
    </xsl:template>

    <xsl:template match="defect" priority="1.5">
        <p><a
                href="http://sourceforge.net/tracker/index.php?func=detail&amp;aid={@id}&amp;group_id=29872&amp;atid=397617"
                target="_new" class="bodylink">
                <xsl:value-of select="@id"/>
            </a>&#xa0; <xsl:apply-templates/>
        </p>
    </xsl:template>

    <!-- add <p> elements inside a <td> or <li> if not there already -->

    <xsl:template match="td[not(p) and not(@align='center')]">
        <!-- the test on "align" is to avoid changing the knight's tour diagram -->
        <xsl:copy copy-namespaces="no">
            <!--
<xsl:copy-of select="@*"/>
-->
            <xsl:apply-templates select="@*"/>
            <p>
                <xsl:apply-templates/>
            </p>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="li[not(p)]">
        <xsl:copy copy-namespaces="no">
            <!--
<xsl:copy-of select="@*"/>
-->
            <xsl:apply-templates select="@*"/>
            <p>
                <xsl:apply-templates/>
            </p>
        </xsl:copy>
    </xsl:template>

    <!--<xsl:template match="fn:conformance">
<xsl:variable name="temp">
<xsl:document validation="strict">
<xsl:apply-templates select="fn:functions" mode="f"/>
</xsl:document>
</xsl:variable>
<xsl:apply-templates select="$temp//body"/>
</xsl:template>-->

    <!-- Generate list of functions in main table of contents -->
    <xsl:template name="make-function-contents-page">
        <xsl:variable name="fns"
            select="conformance/functions/function[starts-with(name, 'fn:') or starts-with(name, 'xslt:')]"/>
        <xsl:variable name="sfns" as="element()*">
            <xsl:perform-sort select="$fns[. is key('fk', name)[1]]">
                <xsl:sort select="substring-after(name, ':')"/>
            </xsl:perform-sort>
        </xsl:variable>
        <ul>
            <xsl:for-each select="$sfns">
                <li>
                    <p>
                        <a href="functions/intro/{translate(name, ':', '_')}.html">
                            <xsl:value-of select="name"/>
                        </a>
                    </p>
                </li>
            </xsl:for-each>
        </ul>
    </xsl:template>

    <xsl:template match="html-example">
        <xsl:copy-of select="*"/>
    </xsl:template>

    <xsl:template match="processing-instruction(featurekeys)">
        <xsl:apply-templates select="doc('../src/FeatureKeysDoc.xml')/*"/>
    </xsl:template>


</xsl:stylesheet>
