<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:c="http://www.saxonica.com/ns/doc/functions"
    exclude-result-prefixes="xs"
    version="2.0">
    
    <!-- further processing of the function library document to give it an article/section structure -->
    
    <xsl:output method="xml" indent="yes"/>
    
    <xsl:function name="c:usual-prefix" as="xs:string">
        <xsl:param name="ns" as="xs:string"/>
        <xsl:choose>
            <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions'">fn</xsl:when>
            <xsl:when test="$ns = 'http://saxon.sf.net/'">saxon</xsl:when>
            <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions/map'">map</xsl:when>
            <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions/math'">math</xsl:when>
            <xsl:when test="$ns = 'http://expath.org/ns/file'">epath-file</xsl:when>
            <xsl:when test="$ns = 'http://expath.org/ns/zip'">expath-zip</xsl:when>
            <xsl:when test="$ns = 'http://exslt.org/common'">exslt-common</xsl:when>
            <xsl:when test="$ns = 'http://exslt.org/random'">exslt-random</xsl:when>
            <xsl:when test="$ns = 'http://exslt.org/math'">exslt-math</xsl:when>
            <xsl:otherwise><xsl:value-of select="$ns"/></xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    
    <xsl:template match="c:functions">
        <article id="functions" title="Function Library">
            <h1>Function Library</h1>
            
            <p>This section of the documentation provides a complete list of functions implemented in the
            Saxon function library, organized by namespace.</p>
            
            <p>Hover over a listed function to view a brief description. For full details, select the
                function entry.</p>
            
            <xsl:for-each-group select="c:function" group-by="c:name/@namespace">
                <xsl:sort select="if (c:usual-prefix(current-grouping-key()) = 'fn') then '' else c:usual-prefix(current-grouping-key())"/>
                <section id="{c:usual-prefix(current-grouping-key())}" title="Namespace {c:usual-prefix(current-grouping-key())}">
                    <h1>Namespace <xsl:value-of select="c:usual-prefix(current-grouping-key())"/></h1>
                    <xsl:for-each select="current-group()">
                        <xsl:sort select="lower-case(c:name)"/>
                        <xsl:copy-of select="." copy-namespaces="no"/>
                    </xsl:for-each>
                </section>
            </xsl:for-each-group>
        </article>        
    </xsl:template>
</xsl:stylesheet>