<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:el="http://www.saxonica.com/ns/doc/elements"
    xmlns:e="http://www.w3.org/1999/XSL/Spec/ElementSyntax"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:map="http://www.w3.org/2005/xpath-functions/map"
    xmlns:doc="http://www.saxonica.com/ns/doc" exclude-result-prefixes="xs el e xsi map"
    version="3.0">

    <xsl:output method="xml" indent="yes"/>

    <xsl:variable name="catalog" select="doc(resolve-uri('element-catalog.xml', base-uri(/)))"
        doc:doc="The W3C Spec catalog that contains the syntax information for the elements in this section"/>
    <xsl:variable name="namespace" select="'http://www.w3.org/1999/XSL/Transform'"
        doc:doc="The namespace for these elements"/>

    <xsl:template match="article">
        <xsl:copy>
            <xsl:sequence select="@id|@title"/>
            <xsl:comment>Generated - <xsl:value-of select="current-dateTime()"/> from src/elements/xsl-elements-src.xml (the editable version)</xsl:comment>
            <xsl:apply-templates select="*" mode="#current"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="@*|*|text()">
        <xsl:copy>
            <xsl:apply-templates select="node()|@*" mode="#current"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="section">
        <xsl:copy>
            <xsl:sequence select="@id|@title"/>
            <!--<xsl:apply-templates select="el:xsl-element" mode="#current">
                <xsl:sort select="/name"/>
            </xsl:apply-templates>-->
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="el:xsl-element"
        xpath-default-namespace="http://www.saxonica.com/ns/doc/elements">
        <xsl-element xmlns="http://www.saxonica.com/ns/doc/elements">
            <xsl:variable name="atts"
                select="map:merge((attributes/attribute ! map:entry(xs:string(./@name),./(*|text()))))" as="map(xs:string,node()*)"/>
            <name namespace="{$namespace}">
                <xsl:value-of select="name"/>
            </name>
            <xsl:apply-templates select="$catalog//e:element-syntax[@name=current()/name]">
                <xsl:with-param name="atts" select="$atts" tunnel="yes"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="*" mode="#current"/>
        </xsl-element>
    </xsl:template>

    <xsl:template match="el:name|el:attributes"/>

    <xsl:template match="e:attribute">
        <xsl:param name="atts" as="map(xs:string,node()*)" tunnel="yes"/>
        <attribute xmlns="http://www.saxonica.com/ns/doc/elements">
            <xsl:apply-templates select="@*" mode="#current"/>
            <desc>
                <xsl:sequence select="$atts(@name)"/>
            </desc>
            <xsl:apply-templates select="*" mode="#current"/>
        </attribute>
    </xsl:template>

    <xsl:template match="e:*">
        <xsl:element name="{local-name(.)}" namespace="http://www.saxonica.com/ns/doc/elements">
            <xsl:apply-templates select="@*|*|text()" mode="#current"/>
        </xsl:element>
    </xsl:template>

    <xsl:template match="e:element">
        <content xmlns="http://www.saxonica.com/ns/doc/elements">
            <xsl:apply-templates select="@*|*|text()" mode="#current"/>
        </content>
    </xsl:template>


</xsl:stylesheet>
