<xsl:stylesheet id="ss" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
 xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs fc"
 xmlns:style="http://saxonica.com/ns/html-style-property"
 xmlns:ixsl="http://saxonica.com/ns/interactiveXSLT" extension-element-prefixes="ixsl"
 xmlns:prop="http://saxonica.com/ns/html-property"
 xmlns:fc="http://www.saxonica.com/ns/doc/functions" xmlns:doc="http://www.saxonica.com/ns/doc">

 <xsl:variable name="implementation-doc" select="doc('../doc/implement.xml')" as="document-node()"/>


 <xsl:template match="xsl:stylesheet" mode="f"/>

 <!-- Description in the hover box. -->
 <xsl:template match="fc:description" mode="fn-description">
  <div>
   <xsl:apply-templates mode="f"/>
   <!--   <xsl:copy-of select="child::node()"/>-->
  </div>
 </xsl:template>

 <!-- Description in the hover box, when there are multiple signatures. -->
 <xsl:template match="fc:signatures" mode="fn-description">
  <xsl:if test="fc:proto/fc:description">
   <ol>
    <xsl:for-each select="fc:proto/fc:description">
     <li>
      <!--      <xsl:copy-of select="."/>-->
      <xsl:apply-templates mode="f"/>
     </li>
    </xsl:for-each>
   </ol>
  </xsl:if>
 </xsl:template>

 <!-- Produce sorted list of functions. -->
 <xsl:function name="fc:fn-list" as="element()*">
  <xsl:param name="fns" as="element()*"/>
  <xsl:perform-sort select="$fns">
   <xsl:sort select="lower-case(fc:name)" lang="en"/>
  </xsl:perform-sort>
 </xsl:function>

 <!-- Get function namespace 'usual' (conventional) prefix (used for definition page titles and function references). -->
 <xsl:function name="fc:usual-prefix" as="xs:string">
  <xsl:param name="ns" as="xs:string"/>
  <xsl:choose>
   <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions'">fn</xsl:when>
   <xsl:when test="$ns = 'http://saxon.sf.net/'">saxon</xsl:when>
   <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions/map'">map</xsl:when>
   <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions/math'">math</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/archive'">arch</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/binary'">bin</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/file'">file</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/zip'">zip</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/common'">exsl</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/random'">random</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/math'">exslt-math</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/sets'">set</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/dates-and-times'">date</xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$ns"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:function>
 
 <!-- Get function namespace reference (short form name used in URL page names). -->
 <xsl:function name="fc:namespace-ref" as="xs:string">
  <xsl:param name="ns" as="xs:string"/>
  <xsl:choose>
   <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions'">fn</xsl:when>
   <xsl:when test="$ns = 'http://saxon.sf.net/'">saxon</xsl:when>
   <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions/map'">map</xsl:when>
   <xsl:when test="$ns = 'http://www.w3.org/2005/xpath-functions/math'">math</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/archive'">expath-archive</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/binary'">expath-binary</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/file'">expath-file</xsl:when>
   <xsl:when test="$ns = 'http://expath.org/ns/zip'">expath-zip</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/common'">exslt-common</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/random'">exslt-random</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/math'">exslt-math</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/sets'">exslt-sets</xsl:when>
   <xsl:when test="$ns = 'http://exslt.org/dates-and-times'">exslt-dates-and-times</xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$ns"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:function>

 <!-- Function Library main page. -->
 <xsl:template match="article" mode="function-list">
  <xsl:variable name="title" select="(h1|h2|h3)[1]"/>
  <xsl:if test="exists($title)">
   <ixsl:set-property name="document.title" select="concat('Saxon ', $title)"/>
  </xsl:if>
  <h1>Function Library</h1>
  <p>This section of the documentation provides a complete list of functions implemented in the
   Saxon function library, organized by namespace. Click on a namespace to see the list of functions
   in that namespace.</p>
  <table class="list">
   <xsl:for-each select="section">
    <xsl:variable name="ns-ref" select="fc:namespace-ref(fc:function[1]/fc:name/@namespace)"/>
    <tr>
     <td><span class="link" data-href="/functions/{$ns-ref}"><xsl:value-of select="$ns-ref"
     /></span></td>
     <td><p><xsl:value-of select="fc:function[1]/fc:name/@namespace"/></p></td>
    </tr>
   </xsl:for-each>
  </table>
  <p>For further general information about using functions not in the standard function namespace,
   see <span class="link" data-href="/extensions/functions">Extension functions</span>. </p>
 </xsl:template>

 <!-- Namespace main page. -->
 <xsl:template match="section" mode="function-list">
  <xsl:variable name="title" select="(h1|h2|h3)[1]"/>
  <xsl:if test="exists($title)">
   <ixsl:set-property name="document.title" select="concat('Saxon ', $title)"/>
  </xsl:if>
  <article id="functions" title="{@title}">

   <!-- Default content of hover box. -->
   <div id="fn-desc">
    <h4>Functions in namespace <xsl:value-of select="fc:function[1]/fc:name/@namespace"/></h4>
    <p>Hover over a listed function to view a brief description. For full details, select the
     function entry.</p>
   </div>

   <!-- div for namespace functions list, on main page (scrollable section). -->
   <xsl:variable xpath-default-namespace="" name="height"
    select="ixsl:page()/html/body/div[@id = 'wrap']/div[@id = 'main']/@prop:offsetHeight - 140"/>

   <div style="overflow:auto; height:{$height}px; margin-top:5px;margin-bottom:5px">

    <!-- Namespace functions list, on main page. -->
    <xsl:variable name="sfns" as="element()*" select="fc:fn-list(fc:function)"/>

    <p>Number of functions: <xsl:value-of select="count($sfns)"/></p>

    <xsl:variable name="colsize" select="xs:integer(ceiling(count($sfns) div 3))"/>

    <table width="100%">
     <tr>
      <td valign="top">
       <p>
        <xsl:apply-templates select="$sfns[(position()-1) idiv $colsize eq 0]" mode="index"/>
       </p>
      </td>
      <td valign="top">
       <p>
        <xsl:apply-templates select="$sfns[(position()-1) idiv $colsize eq 1]" mode="index"/>
       </p>
      </td>
      <td valign="top">
       <p>
        <xsl:apply-templates select="$sfns[(position()-1) idiv $colsize eq 2]" mode="index"/>
       </p>
      </td>
     </tr>
    </table>

   </div>

   <!-- Implementation note at the bottom of namespace page. -->
   <p class="small-note">The information in this section indicates which functions are implemented
    in this Saxon release, and any restrictions in the current implementation. It includes both the
    core functions defined in XPath, and the additional functions defined in the XSLT
    specification.</p>
   <!--
    <xsl:apply-templates select="$sfns" mode="f"/>
   -->

  </article>
 </xsl:template>

 <!-- Contents of namespace function list: links to each function in namespace. -->
 <xsl:template match="fc:function" mode="index">
  <span class="flink" data-href="{fc:name}" data-ns="{fc:namespace-ref(fc:name/@namespace)}">
   <xsl:value-of select="fc:name"/>
  </span>
  <br/>
 </xsl:template>


 <!-- This doesn't seem to be used in the viewer-app section of the website. DL 2014-06-06 -->

 <!--<xsl:template match="fc:functions" mode="f">
  
  <article id="functions" title="Function Library">

   <div id="fn-desc">
    <h4>Function Library</h4>
    <p>Hover over a listed function to view a brief description. For full details, select the
     function entry.</p>
   </div>

   <xsl:variable xpath-default-namespace="" name="height"
    select="ixsl:page()/html/body/div[@id = 'wrap']/div[@id = 'main']/@prop:offsetHeight - 140"/>

   <div style="overflow:auto; height:{$height}px; margin-top:5px;margin-bottom:5px">

    <xsl:for-each-group select="function" group-by="name/@namespace">
     <h1>Namespace <xsl:value-of select="fc:usual-prefix(current-grouping-key())"/>
      <small>&#xa0;&#xa0;&#xa0;(<xsl:value-of select="current-grouping-key()"/>)</small></h1>
     <!-\- <xsl:variable name="fns" select="function[starts-with(name, 'fn:') or starts-with(name, 'xslt:')]"/> -\->
     <xsl:variable name="sfns" as="element()*" select="fc:fn-list(current-group())"/>

     <xsl:variable name="colsize" select="xs:integer(ceiling(count($sfns) div 3))"/>

     <table>
      <tr>
       <td valign="top">
        <p>
         <xsl:apply-templates select="$sfns[(position()-1) idiv $colsize eq 0]" mode="index"/>
        </p>
       </td>
       <td valign="top">
        <p>
         <xsl:apply-templates select="$sfns[(position()-1) idiv $colsize eq 1]" mode="index"/>
        </p>
       </td>
       <td valign="top">
        <p>
         <xsl:apply-templates select="$sfns[(position()-1) idiv $colsize eq 2]" mode="index"/>
        </p>
       </td>
      </tr>
     </table>

    </xsl:for-each-group>

   </div>

   <p class="small-note">The information in this section indicates which functions are implemented
    in this Saxon release, and any restrictions in the current implementation. It includes both the
    core functions defined in XPath, and the additional functions defined in the XSLT
    specification.</p>
   <!-\-
<xsl:apply-templates select="$sfns" mode="f"/>
-\->
  </article>
 </xsl:template>-->

 <!--  Function pages for each function. -->
 <!--  (Previously just for those in namespace 'fn'. [name/@namespace = 'http://www.w3.org/2005/xpath-functions'] 2014-07-16)  -->
 <xsl:template match="fc:function" priority="5" mode="f">

  <xsl:variable name="MyFunction" select="."/>
  <xsl:variable name="namespace" select="ancestor::section/@id"/>
  <xsl:variable name="default.saxon"
   select="$implementation-doc/implement/default[@id = $namespace]/@saxon"/>

  <ixsl:set-property name="document.title"
   select="concat('Saxon ', fc:namespace-ref(fc:name/@namespace),':', fc:name)"/>

  <h1>
   <xsl:value-of select="concat(fc:usual-prefix(fc:name/@namespace),':',fc:name)"/>
  </h1>

  <xsl:apply-templates select="fc:description" mode="add-p-if-necessary"/>

  <!-- Determine if all signatures of the function appear in the same set of versions of specifications. 
       If so, just one 'Specs versions & Saxon editions' note, otherwise one per signature. -->
  <xsl:variable name="same-spec-references" as="xs:boolean"
   select="every $proto in subsequence(fc:signatures/fc:proto, 2)
   satisfies deep-equal(fc:signatures/fc:proto[1]/fc:in-spec, $proto/fc:in-spec)"/>

  <xsl:if test="$same-spec-references">

   <aside>
    <xsl:if test=".//fc:in-spec">
     <xsl:value-of select="fc:specs(fc:signatures/fc:proto[1]/fc:in-spec)"/>
    </xsl:if>
    <xsl:value-of select="fc:edition((fc:saxon-edition,$default.saxon,'default')[1])"/>
   </aside>

  </xsl:if>

  <xsl:apply-templates select="fc:signatures" mode="f">
   <xsl:with-param name="output-spec-references" select="not($same-spec-references)"/>
  </xsl:apply-templates>

  <h3 class="subtitle">Namespace</h3>
  <p>
   <xsl:value-of select="$MyFunction/fc:name/@namespace"/>
  </p>

  <xsl:if test=".//fc:in-spec">
   <h3 class="subtitle">Links to <xsl:if
     test="contains($MyFunction/fc:name/@namespace, 'xpath-functions')">
     <xsl:text>W3C </xsl:text></xsl:if>specifications</h3>
   <!-- Standardize and simplify this. Use functions? DL 2014-07-16 -->
   <xsl:variable name="fname" select="$MyFunction/fc:name"/>
   <xsl:variable name="p-list" select="fc:signatures/fc:proto"/>
   <xsl:variable name="all-spec">
    <xsl:value-of select="for $i in $p-list/fc:in-spec return $i"/>
   </xsl:variable>

   <xsl:for-each-group select="$p-list" group-by="fc:in-spec">
    <xsl:sort select="current-grouping-key()"/>
    <xsl:variable name="p" select="current-grouping-key()"/>

    <xsl:choose>
     <xsl:when test="$MyFunction/fc:name/@namespace = 'http://www.w3.org/2005/xpath-functions/math'">
      <xsl:if test="$p='xpath20'">
       <p>
        <a href="http://www.w3.org/TR/xpath-functions/#func-math-{$fname}">XPath 2.0 Functions and
         Operators</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='xpath30'">
       <p>
        <a href="http://www.w3.org/TR/xpath-functions-30/#func-math-{$fname}">XPath 3.0 Functions
         and Operators</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='xslt20'and not(contains($all-spec, 'xpath20'))">
       <p>
        <a href="http://www.w3.org/TR/xslt20/#function-math-{$fname}">XSLT 2.0 Specification</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='xslt30' and not(contains($all-spec, 'xpath30'))">
       <p>
        <a href="http://www.w3.org/TR/xslt-30/#function-math-{$fname}">XSLT 3.0 Specification</a>
       </p>
      </xsl:if>
     </xsl:when>
     <xsl:when test="contains($MyFunction/fc:name/@namespace, 'expath.org')">
      <xsl:if test="$p='expath-archive'"> 
       <xsl:choose>
        <xsl:when test="$fname='extract-map'">
         <p>
          <a href="http://www.expath.org/spec/archive/editor#fn.extractmap">EXPath Archive Module</a>
         </p>
        </xsl:when>
        <xsl:when test="$fname='entry-names'">
         <p>
          <a href="http://www.expath.org/spec/archive/editor#fn.entry.names.map">EXPath Archive Module</a>
         </p>
        </xsl:when>
        <xsl:otherwise>
         <p>
          <a href="http://www.expath.org/spec/archive/editor#fn.{replace($fname,'-','.')}">EXPath Archive Module</a>
         </p>
        </xsl:otherwise>
       </xsl:choose>           
      </xsl:if>
      <xsl:if test="$p='expath-binary'">
       <p>
        <a href="http://www.expath.org/spec/binary#{$fname}">EXPath Binary Module</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='expath-file'">
       <p>
        <a href="http://www.expath.org/spec/file#{$fname}">EXPath File Module</a>
       </p>
      </xsl:if>
     </xsl:when>
     <xsl:when test="contains($MyFunction/fc:name/@namespace, 'exslt.org')">
      <xsl:if test="$p='exslt-common'">
       <p>
        <a href="http://www.exslt.org/exsl/functions/{$fname}/index.html">EXSLT Common Module</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='exslt-math'">
       <p>
        <a href="http://www.exslt.org/math/functions/{$fname}/index.html">EXSLT Math Module</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='exslt-random'">
       <p>
        <a href="http://www.exslt.org/random/functions/{$fname}/index.html">EXSLT Random Module</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='exslt-sets'">
       <p>
        <a href="http://www.exslt.org/set/functions/{$fname}/index.html">EXSLT Sets Module</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='exslt-dates-and-times'">
       <p>
        <a href="http://www.exslt.org/date/functions/{$fname}/index.html">EXSLT Dates and Times Module</a>
       </p>
      </xsl:if>
     </xsl:when>
     <xsl:otherwise>
      <xsl:if test="$p='xpath20'">
       <p>
        <a href="http://www.w3.org/TR/xpath-functions/#func-{$fname}">XPath 2.0 Functions and
         Operators</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='xpath30'">
       <p>
        <a href="http://www.w3.org/TR/xpath-functions-30/#func-{$fname}">XPath 3.0 Functions and
         Operators</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='xslt20' and not(contains($all-spec, 'xpath20'))">
       <p>
        <a href="http://www.w3.org/TR/xslt20/#function-{$fname}">XSLT 2.0 Specification</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='xslt30' and not(contains($all-spec, 'xpath30'))">
       <p>
        <a href="http://www.w3.org/TR/xslt-30/#function-{$fname}">XSLT 3.0 Specification</a>
       </p>
      </xsl:if>
      <xsl:if test="$p='xquery-update10'">
       <p>
        <a href="http://www.w3.org/TR/xquery-update-10/#id-func-{$fname}">XQuery Update 1.0</a>
       </p>
      </xsl:if>
     </xsl:otherwise>
    </xsl:choose>

   </xsl:for-each-group>
  </xsl:if>

  <xsl:apply-templates select="fc:status" mode="f"/>
  <xsl:apply-templates select="fc:notes" mode="f"/>
  <xsl:apply-templates select="fc:details" mode="f"/>

  <xsl:if test="fc:see-also">
   <h3 class="subtitle">See also:</h3>
   <xsl:apply-templates select="fc:see-also" mode="f"/>
  </xsl:if>

 </xsl:template>

 <xsl:template match="fc:description[p]" mode="add-p-if-necessary">
  <xsl:apply-templates mode="f"/>
 </xsl:template>

 <xsl:template match="fc:description" mode="add-p-if-necessary">
  <p>
   <xsl:apply-templates mode="f"/>
  </p>
 </xsl:template>

 <!-- Signature formats. -->
 <xsl:template match="signatures" mode="f"
  xpath-default-namespace="http://www.saxonica.com/ns/doc/functions">
  <xsl:param name="output-spec-references" as="xs:boolean"/>

  <xsl:for-each select="proto">
   <xsl:variable name="MyProto" select="."/>

   <p class="fn-sig">
    <xsl:value-of
     select="concat(../../name,'(',string-join(for $arg in arg return concat('$',$arg/@name, ' as ', $arg/@type), ', ') ,')', ' &#x2794;  ',@return-type)"
    />
   </p>
   <xsl:apply-templates select="description" mode="add-p-if-necessary"/>
   <xsl:if test="$output-spec-references">
    <xsl:variable name="namespace" select="ancestor::section/@id"/>
    <xsl:variable name="default.saxon"
     select="$implementation-doc/implement/default[@id = $namespace]/@saxon"/>

    <aside>
     <xsl:value-of select="fc:specs(in-spec)"/>
     <xsl:value-of select="fc:edition((../../saxon-edition,$default.saxon,'default')[1])"/>
    </aside>


   </xsl:if>
   <table class="fn-prototype" style="margin-bottom:10px;">
    <tr>
     <td width="470" align="left" colspan="4" style="border-top:solid 1px;">
      <p>
       <i>
        <xsl:value-of select="if (exists(arg)) then 'Arguments' else 'There are no arguments'"/>
       </i>
      </p>
     </td>
    </tr>
    <xsl:for-each select="arg">
     <tr>
      <td width="40">
       <p>&#xa0;</p>
      </td>
      <td width="80" valign="top">
       <p>$<xsl:value-of select="@name"/></p>
      </td>
      <td valign="top" width="150">
       <p>
        <xsl:value-of select="@type"/>
       </p>
      </td>
      <td valign="top" width="200">
       <p>
        <xsl:value-of select="@desc"/>
       </p>
      </td>
     </tr>
    </xsl:for-each>
    <tr>
     <td colspan="2" style="border-top:solid 1px; border-bottom:solid 1px;">
      <p>
       <i>Result</i>
      </p>
     </td>

     <td style="border-top:solid 1px #3D5B96; border-bottom:solid 1px;" colspan="2">
      <p>
       <xsl:value-of select="$MyProto/@return-type"/>
      </p>
     </td>

    </tr>
   </table>

  </xsl:for-each>
 </xsl:template>

 <!-- See also section with links to other related functions. -->
 <xsl:template match="fc:see-also" mode="f">
  <xsl:variable name="prefix" select="fc:usual-prefix(@namespace)"/>
  <xsl:variable name="ns-ref" select="fc:namespace-ref(@namespace)"/>
  <p>
   <span class="link" data-href="/functions/{$ns-ref}/{@name}"><xsl:value-of
     select="concat($prefix, ':', @name)"/>()</span>
  </p>
 </xsl:template>

 <!-- Commented out, and link to this template removed. Don't bother with default messages. DL 2014-07-18 -->
 <!--<xsl:template match="fc:status-ok[../fc:name/@namespace='http://www.w3.org/2005/xpath-functions']"
  mode="f">
  <!-\-<xsl:if test="../fc:name[@namespace='http://www.w3.org/2005/xpath-functions']">-\->
  <h3 class="subtitle">Notes on the Saxon implementation</h3>
  <p><!-\-<xsl:value-of select="$implementation-doc/implement/saxon[@version = 'ALL']"/> -\-> The
   function is fully implemented according to the W3C specifications.</p>
  <!-\-</xsl:if>-\->
 </xsl:template>-->

 <!--<xsl:template match="fc:status-ok" mode="f"> <!-\- Currently never hit this. DL 2014-07-18 -\->
  <h3 class="subtitle">Notes on the Saxon implementation</h3>
  <p>
   <xsl:value-of select="$implementation-doc/implement/saxon[@version = 'PE']"/>
  </p>
 </xsl:template>-->

 <!-- <fc:status-ok> elements are ignored -->

 <!-- 'Notes on the Saxon implementation' section. -->
 <xsl:template match="fc:status" mode="f">
  <h3 class="subtitle">Notes on the Saxon implementation</h3>

  <!-- This is now all output in message at the beginning of function template. DL 2014-07-18
   <xsl:variable name="namespace" select="ancestor::section/@id"/>
  <xsl:variable name="default.saxon"
   select="$implementation-doc/implement/default[@id = $namespace]/@saxon"/>
  <p>
   <xsl:value-of select="fc:edition((../fc:saxon-edition,$default.saxon,'default')[1])"/>
   <xsl:value-of select="fc:specs(..//fc:in-spec[1])"/>
  </p>-->
  <xsl:apply-templates mode="f"/>
 </xsl:template>

 <xsl:template match="fc:details" mode="f">
  <h3 class="subtitle">Details</h3>
  <xsl:apply-templates mode="f"/>
 </xsl:template>

 <xsl:template match="fc:notes" mode="f">
  <h3 class="subtitle">Notes</h3>
  <xsl:apply-templates mode="f"/>
  <!--<p>
            <b>Note:</b>
            <xsl:apply-templates mode="f"/>
        </p>-->
 </xsl:template>


 <!-- What do these templates do? Are they used? DL 2014-07-16-->
 <xsl:template match="div" xpath-default-namespace="" priority="8" mode="function-doc">
  <xsl:apply-templates select="child::node()" mode="function-doc"/>
 </xsl:template>

 <xsl:template match="*[namespace-uri()='']" priority="7" mode="function-doc">
  <xsl:copy-of select="."/>
 </xsl:template>

 <!--
<xsl:template match="status" mode="f">
<xsl:copy-of select="child::node()"/>
</xsl:template>

<xsl:template match="status[p]" mode="f">
<xsl:copy-of select="child::node()"/>
</xsl:template>
-->

 <!-- Determine text within 'Specs versions & Saxon editions' note: 
  Spec versions in which the functions are defined; or for 'expath' functions, the xpath requirement information. -->
 <xsl:function name="fc:specs" as="xs:string*">
  <xsl:param name="in" as="element(fc:in-spec)*"/>
  <xsl:variable name="my-function" as="element(fc:function)" select="$in/ancestor::fc:function"/>
  <xsl:variable name="namespace" select="$in/ancestor::section/@id"/>
  <xsl:variable name="default.saxon"
   select="$implementation-doc/implement/default[@id = $namespace]/@saxon"/>
  <xsl:variable name="default.xpath"
   select="$implementation-doc/implement/default[@id = $namespace]/@xpath"/>
  <xsl:choose>
   <!-- Get XPath requirement information for functions in 'expath' namespace. -->
   <xsl:when test="contains($in[1], 'expath')">
    <!--    <xsl:variable name="xpath-version" select="$my-function/fc:xpath"/>-->
    <xsl:value-of
     select="$implementation-doc/implement/xpath[@version = ($my-function/fc:xpath, $default.xpath)[1]]"
    />
   </xsl:when>
   <!-- No XPath or XSLT requirement information for functions in 'exslt' namespace. -->
   <xsl:when test="contains($in[1], 'exslt')"> </xsl:when>
   <xsl:otherwise>
    <!-- in-spec elements should be in order such that first one $in[1] is earliest spec, to ensure the correct message. -->
    <xsl:value-of select="$implementation-doc/implement/spec[@value = $in[1]]"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:function>

 <!-- Determine text within 'Specs versions & Saxon editions' note: Saxon editons & versions. -->
 <xsl:function name="fc:edition" as="xs:string*">
  <xsl:param name="available" as="xs:string"/>
  <xsl:variable name="available"
   select="for $t in tokenize($available, '\s+') return upper-case($t)" as="xs:string*"/>
  <xsl:for-each select="$available">
   <xsl:value-of select="$implementation-doc/implement/saxon[@version = current()]"/>
  </xsl:for-each>

 </xsl:function>

</xsl:stylesheet>
